"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFlow = function () {
    var flowObjects = {};
    return {
        get: function (name, fallback) {
            var flowObject = flowObjects[name];
            if (flowObjects.hasOwnProperty(name))
                return flowObject;
            if (typeof fallback !== 'undefined')
                return fallback;
            throw new Error("cannot get flow[\"" + name + "\"], no fallback provided.");
        },
        getAll: function () {
            return flowObjects;
        },
        setMultiple: function (setObj) {
            var _this = this;
            Object.entries(setObj).forEach(function (_a) {
                var name = _a[0], value = _a[1];
                return _this.set(name, value);
            });
        },
        set: function (name, value) {
            if (flowObjects.hasOwnProperty(name))
                throw new Error("cannot set flow[\"" + name + "\"], value is already set.");
            flowObjects[name] = value;
        },
        update: function (name, updater, defaultValue) {
            var currentValue = typeof flowObjects[name] === "undefined" ? defaultValue : flowObjects[name];
            flowObjects[name] = updater(currentValue);
        },
        teardown: function () {
            Object.keys(flowObjects).forEach(function (key) { return delete flowObjects[key]; });
        }
    };
};
//# sourceMappingURL=flow-variables.js.map